var map;							//! マップ オブジェクト。
var geo;							//! Geo コード取得用オブジェクト。
var isInitialized = false;			//! 初期化フラグ。
var markers       = new Array();	//! マーカーのコレクション。
var selectedID    = -1;				//! 選択されているマーカーの識別子。

// Marker に id プロパティを追加 ( 初期値は無効値 )
google.maps.Marker.prototype.id = -1;

/**
 * マップの初期化を行います。
 */
function initialize()
{
	var mapdiv    = document.getElementById( "map_canvas" );
	var myOptions = { zoom: 16, center: new google.maps.LatLng( 35.6894876, 139.6917064 ), mapTypeId: google.maps.MapTypeId.ROADMAP, scaleControl: true };
	map           = new google.maps.Map( mapdiv, myOptions );
	geo           = new google.maps.Geocoder();

	// マップの中央位置が更新された時のイベント
	google.maps.event.addListener( map, 'center_changed', function()
	{
		setTimeout( notifyLocation, 200 );
	});

	// マップのタイル読み込みが完了した時のイベント
	google.maps.event.addListener( map, 'tilesloaded', function()
	{
		// 起動時に一度だけ座標を更新する
		if( !isInitialized )
		{
			isInitialized = true;
			notifyLocation();
		}
	});
}

/**
 * マップ上にマーカーを追加します。
 * マーカーの初期位置は、現在表示されているマップの中央座標となります。
 *
 * @param	id			マーカーの識別子。
 * @param	name		マーカーに付ける名前。
 * @param	latitude	緯度。
 * @param	longitude	経度。
 */
function addMarker( id, name, latitude, longitude )
{
	var latlng = new google.maps.LatLng( latitude, longitude );
	var marker = new google.maps.Marker( { position: latlng, map: map, title: name, draggable: true } );
	marker.id = id;

	// 選択
	google.maps.event.addListener( marker, 'click', function()
	{
		selectedID = marker.id;
		window.external.OnMarkerSelected( marker.id );
	});

	// 移動開始
	google.maps.event.addListener( marker, 'dragstart', function()
	{
		selectedID = marker.id;
		window.external.OnMarkerSelected( marker.id );
		window.external.OnMarkerMoveStarted( marker.id );
	});

	// 移動中
	google.maps.event.addListener( marker, 'drag', function()
	{
		window.external.OnMarkerMoved( marker.id, marker.position.lat(), marker.position.lng() );
	});

	// 移動終了
	google.maps.event.addListener( marker, 'dragend', function()
	{
		window.external.OnMarkerMoveFinished( marker.id );
	});


	markers[ markers.length ] = marker;
	window.external.OnMarkerAdded( marker.id, marker.position.lat(), marker.position.lng() );
}

/**
 * 指定されたマーカーを選択状態にします。
 *
 * @param	id	選択するマーカーの識別子。
 */
function selectMarker( id )
{
	selectedID = id;
}

/**
 * 選択されているマーカーのインデックスを取得します。
 *
 * @return	マーカーのコレクション内におけるインデックス。
 *			選択されているマーカーが無ければ -1。
 */
function getSelecedMarkerIndex()
{
	return ( selectedID == -1 ? -1 : searchMarker( selectedID ) );
}

/**
 * 指定されたマーカーの差す位置の住所を取得します。
 * 
 * @return	マーカーのコレクション内におけるインデックス。
 *			見つからなかった場合は -1。
 */
function searchMarker( id )
{
	for( var index = 0; index < markers.length; ++index )
	{
		if( markers[ index ].id == id )
		{
			return index;
		}
	}

	return -1;
}

/**
 * 指定されたマーカーのコレクション中の住所を取得します。
 *
 * @param	id	住所を取得するマーカーの識別子。
 */
function getAddress( id )
{
	var index = searchMarker( id );
	if( index == -1 ) { return; }

	var latlng = new google.maps.LatLng(  markers[ index ].position.lat(), markers[ index ].position.lng() );
	
	if( geo )
	{
		geo.geocode( { 'latLng': latlng }, function( results, status )
		{
			if( results && results[ 0 ] )
			{
				// 住所は種別と内容に分かれているが、それらを "[種別]内容" という書式でまとめ、
				// 種別ごとのデータ区切りは ; とする事で、マネージコード側の加工を容易にする。
				//
				var info;
				var address = results[ 0 ].address_components;
				for( i = 0; i < address.length; ++i )
				{
					var types = address[ i ].types;
					for( j = 0; j < types.length; ++j )
					{
						info += "[" + types[ j ] + "]" + address[ i ].long_name + ";";
					}
				}

				// 最後に整形済みの住所を加えておく
				info += "[formatted_address]" + results[ 0 ].formatted_address + ";";
				window.external.OnMarkerAddress( id, info );
			}
		} );
	}
}

/**
 * 選択されているマーカーを削除します。
 */
function removeMarker()
{
	var index = getSelecedMarkerIndex();
	if( index == -1 ) { return; }

	var id = markers[ index ].id;

	markers[ index ].setMap();
	markers[ index ] = null;
	markers.splice( index, 1 );
	selectedID = -1;

	window.external.OnMarkerRemoved( id );
}

/**
 * 現在選択しているマーカーが表示されるようにマップを移動します。
 */
function showMarker()
{
	var index = getSelecedMarkerIndex();
	if( index == -1 ) { return; }

	map.setCenter( markers[ index ].position );
	notifyLocation();
}

/**
 * 現在位置の緯度・経度を取得してマネージ コードへ通知します。
 */
function notifyLocation()
{
	var center = map.getCenter();
	window.external.OnMapMoved( center.lat(), center.lng() );
}

/**
 * 指定された住所の座標へ、マップを移動します。
 *
 * @param	address	住所。
 */
function moveMap( address )
{
	if( geo )
	{
		geo.geocode( { 'address': address }, function( results, status )
		{
			if( results && results[ 0 ] )
			{
				map.setCenter( results[ 0 ].geometry.location );
				notifyLocation();
			}
		});
	}
}

